@@ -1,19 +1,34 @@
-// Copyright (C) 2000-2010 Heiko Bauke <heiko.bauke@mpi-hd.mpg.de>
-//  
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License in
-// version 2 as published by the Free Software Foundation.
-//  
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//  
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-// 02111-1307, USA.
-//  
+// Copyright (c) 2000-2010, Heiko Bauke
+// All rights reserved.
+// 
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions
+// are met:
+// 
+//   * Redistributions of source code must retain the above copyright
+//     notice, this list of conditions and the following disclaimer.  
+// 
+//   * Redistributions in binary form must reproduce the above
+//     copyright notice, this list of conditions and the following
+//     disclaimer in the documentation and/or other materials provided
+//     with the distribution.  
+// 
+//   * Neither the name of the copyright holder nor the names of its
+//     contributors may be used to endorse or promote products derived
+//     from this software without specific prior written permission.
+// 
+// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+// COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
+// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
+// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
+// OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #if !(defined TRNG_BINOMIAL_DIST_HPP)
 
@@ -69,7 +84,7 @@ namespace trng {
       void p(double p_new) { p_=p_new;  calc_probabilities(); }
       int n() const { return n_; }
       void n(int n_new) { n_=n_new;  calc_probabilities(); }
-      explicit param_type(double p, int n) :
+      param_type(double p, int n) :
 	p_(p), n_(n) {
 	calc_probabilities();
       }
@@ -81,7 +96,7 @@ namespace trng {
     
   public:
     // constructor
-    explicit binomial_dist(double p, int n) : P(p, n) {
+    binomial_dist(double p, int n) : P(p, n) {
     }
     explicit binomial_dist(const param_type &P) : P(P) {
     }
@@ -133,7 +148,7 @@ namespace trng {
   }
   inline bool operator!=(const binomial_dist::param_type &p1, 
 			 const binomial_dist::param_type &p2) {
-    return !(p1==p2);
+    return not (p1==p2);
   }
   
   // Streamable concept
