@@ -34,6 +34,7 @@
 
 #define TRNG_EXTREME_VALUE_DIST_HPP
 
+#include <trng/cuda.hpp>
 #include <trng/limits.hpp>
 #include <trng/utility.hpp>
 #include <trng/math.hpp>
@@ -55,12 +56,18 @@ namespace trng {
     private:
       result_type theta_, eta_;
     public:
+      TRNG_CUDA_ENABLE
       result_type theta() const { return theta_; }
+      TRNG_CUDA_ENABLE
       void theta(result_type theta_new) { theta_=theta_new; }
+      TRNG_CUDA_ENABLE
       result_type eta() const { return eta_; }
+      TRNG_CUDA_ENABLE
       void eta(result_type eta_new) { eta_=eta_new; }
+      TRNG_CUDA_ENABLE
       param_type() : theta_(1), eta_(0) {
       }
+      TRNG_CUDA_ENABLE
       param_type(result_type theta, result_type eta) : theta_(theta), eta_(eta) {
       }
 
@@ -106,47 +113,65 @@ namespace trng {
     
   public:
     // constructor
+    TRNG_CUDA_ENABLE
     extreme_value_dist(result_type theta, result_type eta) : p(theta, eta) {
     }
+    TRNG_CUDA_ENABLE
     explicit extreme_value_dist(const param_type &p) : p(p) {
     }
     // reset internal state
+    TRNG_CUDA_ENABLE
     void reset() { }
     // random numbers
     template<typename R>
+    TRNG_CUDA_ENABLE
     result_type operator()(R &r) {
       return p.eta()+p.theta()*math::ln(-math::ln(utility::uniformoo<result_type>(r)));
     }
     template<typename R>
+    TRNG_CUDA_ENABLE
     result_type operator()(R &r, const param_type &p) {
       extreme_value_dist g(p);
       return g(r);
     }
     // property methods
+    TRNG_CUDA_ENABLE
     result_type min() const { return -math::numeric_limits<result_type>::infinity(); }
+    TRNG_CUDA_ENABLE
     result_type max() const { return math::numeric_limits<result_type>::infinity(); }
+    TRNG_CUDA_ENABLE
     param_type param() const { return p; }
+    TRNG_CUDA_ENABLE
     void param(const param_type &p_new) { p=p_new; }
+    TRNG_CUDA_ENABLE
     result_type theta() const { return p.theta(); }
+    TRNG_CUDA_ENABLE
     void theta(result_type theta_new) { p.theta(theta_new); }
+    TRNG_CUDA_ENABLE
     result_type eta() const { return p.eta(); }
+    TRNG_CUDA_ENABLE
     void eta(result_type eta_new) { p.eta(eta_new); }
     // probability density function  
+    TRNG_CUDA_ENABLE
     result_type pdf(result_type x) const {
       x-=p.eta();
       x/=p.theta();
       return math::exp(x-math::exp(x))/p.theta();
     }
     // cumulative density function 
+    TRNG_CUDA_ENABLE
     result_type cdf(result_type x) const {
       x-=p.eta();
       x/=p.theta();
       return 1-math::exp(-math::exp(x));
     }
     // inverse cumulative density function 
+    TRNG_CUDA_ENABLE
     result_type icdf(result_type x) const {
       if (x<=0 or x>=1) {
+#if !(defined __CUDA_ARCH__)
 	errno=EDOM;
+#endif
 	return math::numeric_limits<result_type>::quiet_NaN();
       }
       if (x==0)
@@ -161,12 +186,14 @@ namespace trng {
 
   // EqualityComparable concept
   template<typename float_t>
+  TRNG_CUDA_ENABLE
   inline bool operator==(const typename extreme_value_dist<float_t>::param_type &p1, 
                          const typename extreme_value_dist<float_t>::param_type &p2) {
     return p1.theta()==p2.theta() and p1.eta()==p2.eta();
   }
 
   template<typename float_t>
+  TRNG_CUDA_ENABLE
   inline bool operator!=(const typename extreme_value_dist<float_t>::param_type &p1, 
                          const typename extreme_value_dist<float_t>::param_type &p2) {
     return not (p1==p2);
@@ -176,12 +203,14 @@ namespace trng {
 
   // EqualityComparable concept
   template<typename float_t>
+  TRNG_CUDA_ENABLE
   inline bool operator==(const extreme_value_dist<float_t> &g1, 
 			 const extreme_value_dist<float_t> &g2) {
     return g1.param()==g2.param();
   }
 
   template<typename float_t>
+  TRNG_CUDA_ENABLE
   inline bool operator!=(const extreme_value_dist<float_t> &g1, 
 			 const extreme_value_dist<float_t> &g2) {
     return g1.param()!=g2.param();
