@@ -1,19 +1,34 @@
-// Copyright (C) 2000-2010 Heiko Bauke <heiko.bauke@mpi-hd.mpg.de>
-//  
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License in
-// version 2 as published by the Free Software Foundation.
-//  
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//  
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-// 02111-1307, USA.
-//  
+// Copyright (c) 2000-2010, Heiko Bauke
+// All rights reserved.
+// 
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions
+// are met:
+// 
+//   * Redistributions of source code must retain the above copyright
+//     notice, this list of conditions and the following disclaimer.  
+// 
+//   * Redistributions in binary form must reproduce the above
+//     copyright notice, this list of conditions and the following
+//     disclaimer in the documentation and/or other materials provided
+//     with the distribution.  
+// 
+//   * Neither the name of the copyright holder nor the names of its
+//     contributors may be used to endorse or promote products derived
+//     from this software without specific prior written permission.
+// 
+// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+// COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
+// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
+// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
+// OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #if !(defined TRNG_LAGFIB2PLUS_HPP)
 
@@ -54,9 +69,12 @@ namespace trng {
     class status_type {
       result_type r[utility::ceil2<B>::result];
       unsigned int index;
+      static unsigned int size() {
+	return utility::ceil2<B>::result;
+      }
     public:
       status_type() { 
-	for (unsigned int i=0; i<utility::ceil2<B>::result; ++i)
+	for (unsigned int i=0; i<size(); ++i)
 	  r[i]=0;
 	index=0;
       };
@@ -67,7 +85,7 @@ namespace trng {
       friend bool operator==(const status_type &a, const status_type &b) {
 	if (a.index!=b.index) 
 	  return false;
-	for (unsigned int i=0; i<utility::ceil2<B>::result; ++i)
+	for (unsigned int i=0; i<a.size(); ++i)
 	  if (a.r[i]!=b.r[i])
 	    return false;
 	return true;
@@ -86,7 +104,7 @@ namespace trng {
 		  std::ios_base::left);
 	out << '(' 
 	    << S.index;
- 	for (unsigned int i=0; i<utility::ceil2<B>::result; ++i)
+ 	for (unsigned int i=0; i<S.size(); ++i)
 	  out << ' ' << S.r[i];
 	out << ')';
 	out.flags(flags);
@@ -103,7 +121,7 @@ namespace trng {
 		 std::ios_base::left);
 	in >> utility::delim('(')
 	   >> S_new.index;
- 	for (unsigned int i=0; i<utility::ceil2<B>::result; ++i)
+ 	for (unsigned int i=0; i<S.size(); ++i)
  	  in >> utility::delim(' ') >> S_new.r[i];
 	in >> utility::delim(')');
 	if (in)
