@@ -37,7 +37,9 @@
 #include <ostream>
 #include <istream>
 #include <stdexcept>
+#include <trng/cuda.hpp>
 #include <trng/utility.hpp>
+#include <trng/int_math.hpp>
 
 namespace trng {
   
@@ -47,6 +49,7 @@ namespace trng {
   public:
     // Uniform random number generator concept
     typedef long result_type;
+    TRNG_CUDA_ENABLE
     result_type operator()() const;
   private:
     static const result_type modulus=2147462579l;  // 2^31-21069
@@ -222,12 +225,16 @@ namespace trng {
     }
 
     // Parallel random number generator concept
+    TRNG_CUDA_ENABLE
     void split(unsigned int, unsigned int);
+    TRNG_CUDA_ENABLE
     void jump2(unsigned int);
+    TRNG_CUDA_ENABLE
     void jump(unsigned long long);
 
     // Other useful methods
     static const char * name();
+    TRNG_CUDA_ENABLE
     long operator()(long) const;
 
   private:
@@ -235,12 +242,15 @@ namespace trng {
     mutable status_type S;
     static const char * const name_str;
     
+    TRNG_CUDA_ENABLE
     void backward();
+    TRNG_CUDA_ENABLE
     void step() const;
   };
     
   // Inline and template methods
 
+  TRNG_CUDA_ENABLE
   inline void mrg3s::step() const {
     unsigned long long t(static_cast<unsigned long long>(P.a1)*
 			 static_cast<unsigned long long>(S.r1)+
@@ -248,18 +258,118 @@ namespace trng {
 			 static_cast<unsigned long long>(S.r2)+
 			 static_cast<unsigned long long>(P.a3)*
 			 static_cast<unsigned long long>(S.r3));
-    S.r3=S.r2;  S.r2=S.r1;  S.r1=utility::modulo<modulus, 3>(t);
+    S.r3=S.r2;  S.r2=S.r1;  S.r1=int_math::modulo<modulus, 3>(t);
   }
 
+  TRNG_CUDA_ENABLE
   inline mrg3s::result_type mrg3s::operator()() const {
     step();
     return S.r1;
   }
   
+  TRNG_CUDA_ENABLE
   inline long mrg3s::operator()(long x) const {
     return static_cast<long>(utility::uniformco<double, mrg3s>(*this)*x);
   }
     
+  // Parallel random number generator concept
+  TRNG_CUDA_ENABLE
+  inline void mrg3s::split(unsigned int s, unsigned int n) {
+#if !(defined __CUDA_ARCH__)
+    if (s<1 or n>=s)
+      utility::throw_this(std::invalid_argument("invalid argument for trng::mrg3s::split"));
+#endif
+    if (s>1) {
+      jump(n+1);  long q0=S.r1;
+      jump(s);    long q1=S.r1;
+      jump(s);    long q2=S.r1;
+      jump(s);    long q3=S.r1;
+      jump(s);    long q4=S.r1;
+      jump(s);    long q5=S.r1;
+      long a[3], b[9];
+      a[0]=q3;  b[0]=q2;  b[1]=q1;  b[2]=q0;
+      a[1]=q4;  b[3]=q3;  b[4]=q2;  b[5]=q1;
+      a[2]=q5;  b[6]=q4;  b[7]=q3;  b[8]=q2;
+      int_math::gauss<3>(b, a, modulus);
+      P.a1=a[0];  P.a2=a[1];  P.a3=a[2];
+      S.r1=q2;    S.r2=q1;    S.r3=q0;
+      for (int i=0; i<3; ++i)
+	backward();
+    }
+  }
+  
+  TRNG_CUDA_ENABLE
+  inline void mrg3s::jump2(unsigned int s) {
+    long b[9], c[9], d[3], r[3];
+    long t1(P.a1), t2(P.a2), t3(P.a3);
+    b[0]=P.a1;  b[1]=P.a2;  b[2]=P.a3;
+    b[3]=1l;    b[4]=0l;    b[5]=0l;
+    b[6]=0l;    b[7]=1l;    b[8]=0l;
+    for (unsigned int i(0); i<s; ++i)
+      if ((i&1)==0)
+	int_math::matrix_mult<3>(b, b, c, modulus);
+      else
+	int_math::matrix_mult<3>(c, c, b, modulus);
+    r[0]=S.r1;  r[1]=S.r2;  r[2]=S.r3;
+    if ((s&1)==0)
+      int_math::matrix_vec_mult<3>(b, r, d, modulus);
+    else
+      int_math::matrix_vec_mult<3>(c, r, d, modulus);
+    S.r1=d[0];  S.r2=d[1];  S.r3=d[2];
+    P.a1=t1;    P.a2=t2;    P.a3=t3;
+  }
+
+  TRNG_CUDA_ENABLE
+  inline void mrg3s::jump(unsigned long long s) {
+    if (s<16) {
+      for (unsigned int i(0); i<s; ++i) 
+	step();
+    } else {
+      unsigned int i(0);
+      while (s>0) {
+	if (s%2==1)
+	  jump2(i);
+	++i;
+	s>>=1;
+      }
+    }
+  }
+
+  TRNG_CUDA_ENABLE
+  inline void mrg3s::backward() {
+    long t;
+    if (P.a3!=0l) {
+      t=S.r1;
+      t-=static_cast<long>((static_cast<long long>(P.a1)*
+			    static_cast<long long>(S.r2))%modulus);
+      if (t<0l)
+	t+=modulus;
+      t-=static_cast<long>((static_cast<long long>(P.a2)*
+			    static_cast<long long>(S.r3))%modulus);
+    if (t<0l)
+      t+=modulus;
+    t=static_cast<long>((static_cast<long long>(t)*
+                         static_cast<long long>
+                         (int_math::modulo_invers(P.a3, modulus)))%modulus);
+    } else if (P.a2!=0l) {
+      t=S.r2;
+      t-=static_cast<long>((static_cast<long long>(P.a1)*
+			    static_cast<long long>(S.r3))%modulus);
+      if (t<0l)
+	t+=modulus;
+      t=static_cast<long>((static_cast<long long>(t)*
+			   static_cast<long long>
+			   (int_math::modulo_invers(P.a2, modulus)))%modulus);
+    } else if (P.a1!=0l) {
+      t=S.r3;
+      t=static_cast<long>((static_cast<long long>(t)*
+			   static_cast<long long>
+			   (int_math::modulo_invers(P.a1, modulus)))%modulus);
+    } else
+      t=0l;
+    S.r1=S.r2;  S.r2=S.r3;  S.r3=t;
+  }
+
 }
   
 #endif
