@@ -34,6 +34,7 @@
 
 #define TRNG_RAYLEIGH_DIST_HPP
 
+#include <trng/cuda.hpp>
 #include <trng/limits.hpp>
 #include <trng/utility.hpp>
 #include <trng/math.hpp>
@@ -55,10 +56,14 @@ namespace trng {
     private:
       result_type nu_;
     public:
+      TRNG_CUDA_ENABLE
       result_type nu() const { return nu_; }
+      TRNG_CUDA_ENABLE
       void nu(result_type nu_new) { nu_=nu_new; }
+      TRNG_CUDA_ENABLE
       param_type() : nu_(1) {
       }
+      TRNG_CUDA_ENABLE
       explicit param_type(result_type nu) : nu_(nu) {
       }
 
@@ -103,30 +108,42 @@ namespace trng {
    
   public:
     // constructor
+    TRNG_CUDA_ENABLE
     explicit rayleigh_dist(result_type nu) : p(nu) {
     }
+    TRNG_CUDA_ENABLE
     explicit rayleigh_dist(const param_type &p) : p(p) {
     }
     // reset internal state
+    TRNG_CUDA_ENABLE
     void reset() { }
     // random numbers
     template<typename R>
+    TRNG_CUDA_ENABLE
     result_type operator()(R &r) {
       return icdf(utility::uniformoo<result_type>(r));
     }
     template<typename R>
+    TRNG_CUDA_ENABLE
     result_type operator()(R &r, const param_type &p) {
       rayleigh_dist g(p);
       return g(r);
     }
     // property methods
+    TRNG_CUDA_ENABLE
     result_type min() const { return 0; }
+    TRNG_CUDA_ENABLE
     result_type max() const { return math::numeric_limits<result_type>::infinity(); }
+    TRNG_CUDA_ENABLE
     param_type param() const { return p; }
+    TRNG_CUDA_ENABLE
     void param(const param_type &p_new) { p=p_new; }
+    TRNG_CUDA_ENABLE
     result_type nu() const { return p.nu(); }
+    TRNG_CUDA_ENABLE
     void nu(result_type nu_new) { p.nu(nu_new); }
     // probability density function  
+    TRNG_CUDA_ENABLE
     result_type pdf(result_type x) const {
       if (x<=0)
 	return 0;
@@ -134,13 +151,21 @@ namespace trng {
       return t*math::exp(-t*x/2);
     }
     // cumulative density function 
+    TRNG_CUDA_ENABLE
     result_type cdf(result_type x) const {
       if (x<=0)
 	return 0;
       return 1-math::exp(-x*x/(2*p.nu()*p.nu()));
     }
     // inverse cumulative density function 
+    TRNG_CUDA_ENABLE
     result_type icdf(result_type x) const {
+      if (x<0 or x>1) {
+#if !(defined __CUDA_ARCH__)
+	errno=EDOM;
+#endif
+	return math::numeric_limits<result_type>::quiet_NaN();
+      }
       return p.nu()*math::sqrt(-2*math::ln(1-x));
     }
   };
@@ -149,12 +174,14 @@ namespace trng {
 
   // EqualityComparable concept
   template<typename float_t>
+  TRNG_CUDA_ENABLE
   inline bool operator==(const typename rayleigh_dist<float_t>::param_type &p1, 
 			 const typename rayleigh_dist<float_t>::param_type &p2) {
     return p1.nu()==p2.nu();
   }
 
   template<typename float_t>
+  TRNG_CUDA_ENABLE
   inline bool operator!=(const typename rayleigh_dist<float_t>::param_type &p1, 
 			 const typename rayleigh_dist<float_t>::param_type &p2) {
     return not (p1==p2);
@@ -164,12 +191,14 @@ namespace trng {
 
   // EqualityComparable concept
   template<typename float_t>
+  TRNG_CUDA_ENABLE
   inline bool operator==(const rayleigh_dist<float_t> &g1, 
 			 const rayleigh_dist<float_t> &g2) {
     return g1.param()==g2.param();
   }
 
   template<typename float_t>
+  TRNG_CUDA_ENABLE
   inline bool operator!=(const rayleigh_dist<float_t> &g1, 
 			 const rayleigh_dist<float_t> &g2) {
     return g1.param()!=g2.param();
