@@ -34,6 +34,7 @@
 
 #define TRNG_TRUNCATED_NORMAL_DIST_HPP
 
+#include <trng/cuda.hpp>
 #include <trng/constants.hpp>
 #include <trng/limits.hpp>
 #include <trng/utility.hpp>
@@ -42,7 +43,6 @@
 #include <ostream>
 #include <istream>
 #include <iomanip>
-#include <cerrno>
 
 namespace trng {
 
@@ -56,6 +56,8 @@ namespace trng {
     class param_type {
     private:
       result_type mu_, sigma_, a_, b_, Phi_a, Phi_b;
+
+      TRNG_CUDA_ENABLE
       void update_Phi() {
       if (a_!=-math::numeric_limits<result_type>::infinity())
 	Phi_a=math::Phi((a_-mu_)/sigma_);
@@ -67,20 +69,30 @@ namespace trng {
 	Phi_b=result_type(1);
       }
     public:
+      TRNG_CUDA_ENABLE
       result_type mu() const { return mu_; }
+      TRNG_CUDA_ENABLE
       void mu(result_type mu_new) { mu_=mu_new; update_Phi(); }
+      TRNG_CUDA_ENABLE
       result_type sigma() const { return sigma_; }
+      TRNG_CUDA_ENABLE
       void sigma(result_type sigma_new) { sigma_=sigma_new; update_Phi(); }
-      result_type a() const { return a_; update_Phi(); }
+      TRNG_CUDA_ENABLE
+      result_type a() const { return a_; }
+      TRNG_CUDA_ENABLE
       void a(result_type a_new) { a_=a_new; update_Phi(); }
+      TRNG_CUDA_ENABLE
       result_type b() const { return b_; }
+      TRNG_CUDA_ENABLE
       void b(result_type b_new) { b_=b_new; update_Phi(); }
+      TRNG_CUDA_ENABLE
       param_type() : 
 	mu_(0), sigma_(1), 
 	a_(-math::numeric_limits<result_type>::infinity()), 
 	b_(math::numeric_limits<result_type>::infinity()) {
 	update_Phi(); 
       }
+      TRNG_CUDA_ENABLE
       param_type(result_type mu, result_type sigma, 
 		 result_type a, result_type b) : 
 	mu_(mu), sigma_(sigma), a_(a), b_(b) {
@@ -132,37 +144,55 @@ namespace trng {
    
   public:
     // constructor
+    TRNG_CUDA_ENABLE
     truncated_normal_dist(result_type mu, result_type sigma, 
 			  result_type a, result_type b) : p(mu, sigma, a, b) {
     }
+    TRNG_CUDA_ENABLE
     explicit truncated_normal_dist(const param_type &p) : p(p) {
     }
     // reset internal state
+    TRNG_CUDA_ENABLE
     void reset() { }
     // random numbers
     template<typename R>
+    TRNG_CUDA_ENABLE
     result_type operator()(R &r) {
       return icdf(utility::uniformoo<result_type>(r));
     }
     template<typename R>
+    TRNG_CUDA_ENABLE
     result_type operator()(R &r, const param_type &p) {
       truncated_normal_dist g(p);
       return g(r);
     }
     // property methods
+    TRNG_CUDA_ENABLE
     result_type min() const { return p.a(); }
+    TRNG_CUDA_ENABLE
     result_type max() const { return p.b(); }
+    TRNG_CUDA_ENABLE
     param_type param() const { return p; }
+    TRNG_CUDA_ENABLE
     void param(const param_type &p_new) { p=p_new; }
+    TRNG_CUDA_ENABLE
     result_type mu() const { return p.mu(); }
+    TRNG_CUDA_ENABLE
     void mu(result_type mu_new) { p.mu(mu_new); }
+    TRNG_CUDA_ENABLE
     result_type sigma() const { return p.sigma(); }
+    TRNG_CUDA_ENABLE
     void sigma(result_type sigma_new) { p.sigma(sigma_new); }
+    TRNG_CUDA_ENABLE
     result_type a() const { return p.a(); }
+    TRNG_CUDA_ENABLE
     void a(result_type a_new) { p.a(a_new); }
+    TRNG_CUDA_ENABLE
     result_type b() const { return p.b(); }
+    TRNG_CUDA_ENABLE
     void b(result_type b_new) { p.b(b_new); }
     // probability density function  
+    TRNG_CUDA_ENABLE
     result_type pdf(result_type x) const {
       x-=p.mu();
       x/=p.sigma();
@@ -170,12 +200,14 @@ namespace trng {
 	p.sigma()*math::exp(-0.5*x*x)/(p.Phi_b-p.Phi_a);
     }
     // cumulative density function 
+    TRNG_CUDA_ENABLE
     result_type cdf(result_type x) const {
       x-=p.mu();
       x/=p.sigma();
       return (math::Phi(x)-p.Phi_a)/(p.Phi_b-p.Phi_a);
     }
     // inverse cumulative density function 
+    TRNG_CUDA_ENABLE
     result_type icdf(result_type x) const {
       x*=p.Phi_b-p.Phi_a;
       x+=p.Phi_a;
@@ -187,12 +219,14 @@ namespace trng {
 
   // EqualityComparable concept
   template<typename float_t>
+  TRNG_CUDA_ENABLE
   inline bool operator==(const typename truncated_normal_dist<float_t>::param_type &p1, 
 			 const typename truncated_normal_dist<float_t>::param_type &p2) {
     return p1.mu()==p2.mu() and p1.sigma()==p2.sigma();
   }
 
   template<typename float_t>
+  TRNG_CUDA_ENABLE
   inline bool operator!=(const typename truncated_normal_dist<float_t>::param_type &p1, 
 			 const typename truncated_normal_dist<float_t>::param_type &p2) {
     return not (p1==p2);
@@ -202,12 +236,14 @@ namespace trng {
 
   // EqualityComparable concept
   template<typename float_t>
+  TRNG_CUDA_ENABLE
   inline bool operator==(const truncated_normal_dist<float_t> &g1, 
 			 const truncated_normal_dist<float_t> &g2) {
     return g1.param()==g2.param();
   }
 
   template<typename float_t>
+  TRNG_CUDA_ENABLE
   inline bool operator!=(const truncated_normal_dist<float_t> &g1, 
 			 const truncated_normal_dist<float_t> &g2) {
     return g1.param()!=g2.param();
